@@ -109,21 +109,12 @@ use Exporter;
 		);
 use Carp qw(croak);
 
-use constant USE_ENCODE => ($] >= 5.007003)? 'Encode': '';
+use constant USE_ENCODE => ($] >= 5.008)? 'Encode': '';
 
 my @ENCODE_SUBS = qw(FB_CROAK FB_PERLQQ FB_HTMLCREF FB_XMLCREF
 		     is_utf8 resolve_alias);
 if (USE_ENCODE) {
     eval "use ".USE_ENCODE." \@ENCODE_SUBS;";
-    if ($@) { # Perl 5.7.3 + Encode 0.40
-	eval "use ".USE_ENCODE." qw(is_utf8);";
-	require MIME::Charset::_Compat;
-	for my $sub (@ENCODE_SUBS) {
-	    no strict "refs";
-	    *{$sub} = \&{"MIME::Charset::_Compat::$sub"}
-		unless $sub eq 'is_utf8';
-	}
-    }
 } else {
     require MIME::Charset::_Compat;
     for my $sub (@ENCODE_SUBS) {
@@ -132,7 +123,7 @@ if (USE_ENCODE) {
     }
 }
 
-$VERSION = '1.009.1';
+$VERSION = '1.008';
 
 ######## Private Attributes ########
 
@@ -162,14 +153,13 @@ my %CHARSETS = (# input		    header enc body enc output conv
 		'US-ASCII' =>		[undef,	undef,	undef],
 		'BIG5' =>		['B',	'B',	undef],
 		'GB2312' =>		['B',	'B',	undef],
-		'HZ-GB-2312' =>		['B',	undef,	undef],
 		'EUC-JP' =>		['B',	undef,	'ISO-2022-JP'],
 		'SHIFT_JIS' =>		['B',	undef,	'ISO-2022-JP'],
 		'ISO-2022-JP' =>	['B',	undef,	undef],
 		'KOI8-R' =>		['B',	'B',	undef],
-		'TIS-620' =>		['B',	'B',	undef], # cf. Mew
+		'UTF-8' =>		['S',	'B',	undef],
+		'HZ-GB-2312' =>		['B',	undef,	undef],
 		'UTF-7' =>		['Q',	undef,	undef],
-		'UTF-8' =>		['S',	'S',	undef],
 		'GSM03.38' =>		[undef,	undef,	undef], # not for MIME
 		# We're making this one up to represent raw unencoded 8bit
 		'8BIT' =>		[undef,	'B',	'ISO-8859-1'],
@@ -180,28 +170,19 @@ my %CHARSETS = (# input		    header enc body enc output conv
 my %CHARSET_ALIASES = (# unpreferred		preferred
 		       "ASCII" =>		"US-ASCII",
 		       "BIG5-ETEN" =>		"BIG5",
-		       "CP1250" =>		"WINDOWS-1250",
 		       "CP1251" =>		"WINDOWS-1251",
 		       "CP1252" =>		"WINDOWS-1252",
-		       "CP1253" =>		"WINDOWS-1253",
-		       "CP1254" =>		"WINDOWS-1254",
-		       "CP1255" =>		"WINDOWS-1255",
-		       "CP1256" =>		"WINDOWS-1256",
-		       "CP1257" =>		"WINDOWS-1257",
-		       "CP1258" =>		"WINDOWS-1258",
-		       "CP874" =>		"WINDOWS-874",
 		       "CP936" =>		"GBK",
 		       "CP949" =>		"KS_C_5601-1987",
 		       "EUC-CN" =>		"GB2312",
-		       "HZ" =>			"HZ-GB-2312", # RFC 1842
 		       "KS_C_5601" =>		"KS_C_5601-1987",
 		       "SHIFTJIS" =>		"SHIFT_JIS",
 		       "SHIFTJISX0213" =>	"SHIFT_JISX0213",
-		       "TIS620" =>		"TIS-620", # IANA MIBenum 2259
 		       "UNICODE-1-1-UTF-7" =>	"UTF-7", # RFC 1642 (obs.)
 		       "UTF8" =>		"UTF-8",
 		       "UTF-8-STRICT" =>	"UTF-8", # Perl internal use
-		       "GSM0338" =>		"GSM03.38", # not for MIME
+		       "HZ" =>			"HZ-GB-2312", # RFC 1842
+		       "GSM0338" =>		"GSM03.38",
 		       );
 
 # Some vendors encode characters beyond standardized mappings using extended
@@ -244,7 +225,7 @@ my %ENCODERS = (
 				     ['cp950'],         # Encode::TW
 				     # ['big5-1984',    'Encode::HanExtra'], 
 				    ],
-		    'TIS-620'    => [['cp874'], ],      # Encode::Byte
+		    'TIS620'     => [['iso-8859-11'], ], # Encode::Byte; NBSP
 		    'UTF-8'      => [['utf8'], ],       # Special name on Perl
 		},
 		'STANDARD' => {
@@ -258,11 +239,7 @@ my %ENCODERS = (
 		    'SHIFT_JISX0213'=> [['shiftjisx0213', 'Encode::JIS2K'], ],
 		    'EUC-TW'        => [['euc-tw',      'Encode::HanExtra'], ],
 		    'HZ-GB-2312'    => [['hz'], ],	# Encode::CN
-		    'TIS-620'       => [['tis620'], ],  # (note*)
 		    'GSM03.38'      => [['gsm0338'], ],	# Encode::GSM0338
-
-		    # (note*) ISO-8859-11 was not registered by IANA.
-		    # L<Encode> treats it as canonical name of ``tis-?620''.
 		},
 );
 
@@ -367,22 +344,11 @@ sub new {
     my $class = shift;
     my $charset = shift;
     return bless {}, $class unless $charset;
-    return bless {}, $class if 75 < length $charset; # w/a for CPAN RT #65796.
     my %params = @_;
     my $mapping = uc($params{'Mapping'} || $Config->{Mapping});
 
-    if ($charset =~ /\bhz.?gb.?2312$/i) {
-	# workaround: "HZ-GB-2312" mistakenly treated as "EUC-CN" by Encode
-	# (2.12).
-	$charset = "HZ-GB-2312";
-    } elsif ($charset =~ /\btis-?620$/i) {
-	# workaround: "TIS620" treated as ISO-8859-11 by Encode.
-	# And "TIS-620" not known by some versions of Encode (cf.
-	# CPAN RT #20781).
-	$charset = "TIS-620";
-    } else {
-	$charset = resolve_alias($charset) || $charset
-    }
+    $charset = "HZ" if $charset =~ /\bhz.?gb.?2312$/i; # workaround
+    $charset = resolve_alias($charset) || $charset;
     $charset = $CHARSET_ALIASES{uc($charset)} || uc($charset);
     my ($henc, $benc, $outcset);
     my $spec = $CHARSETS{$charset};
@@ -410,8 +376,6 @@ sub new {
     }, $class;
 }
 
-my %encoder_cache = ();
-
 sub _find_encoder($$) {
     my $charset = uc(shift || "");
     return undef unless $charset;
@@ -419,9 +383,6 @@ sub _find_encoder($$) {
     my ($spec, $name, $module, $encoder);
 
     local($@);
-    $encoder = $encoder_cache{$charset, $mapping};
-    return $encoder if ref $encoder;
-
     foreach my $m (('EXTENDED', 'STANDARD')) {
 	next if $m eq 'EXTENDED' and $mapping ne 'EXTENDED';
 	$spec = $ENCODERS{$m}->{$charset};
@@ -433,13 +394,10 @@ sub _find_encoder($$) {
 		next if $@;
 	    }
 	    $encoder = Encode::find_encoding($name);
-	    last if ref $encoder;
+	    return $encoder if ref $encoder;
 	}
-	last if ref $encoder;
     }
-    $encoder ||= Encode::find_encoding($charset);
-    $encoder_cache{$charset, $mapping} = $encoder if $encoder;
-    return $encoder;
+    return Encode::find_encoding($charset);
 }
 
 =back
@@ -648,7 +606,11 @@ sub body_encode {
             $cset = 'US-ASCII';
         }
     } elsif ($enc eq 'S') {
-	$enc = _resolve_S($encoded, 1);
+	if (_enclen_B($encoded) < _enclen_Q($encoded, 1)) {
+	    $enc = 'B';
+	} else {
+	    $enc = 'Q';
+	}
     } elsif ($enc eq 'B') {
         $enc = 'BASE64';
     } elsif ($enc eq 'Q') {
@@ -683,7 +645,7 @@ Guess 7-bit charset that may encode a string STRING.
 =cut
 
 sub detect_7bit_charset($) {
-    return $DEFAULT_CHARSET unless &USE_ENCODE;
+    return $DEFAULT_CHARSET unless USE_ENCODE;
     my $s = shift;
     return $DEFAULT_CHARSET unless $s;
 
@@ -735,7 +697,7 @@ sub encode($$$;) {
 	$s = $self->{Decoder}->decode($s, ($check & 0x1)? FB_CROAK(): 0);
     }
     my $enc = $self->{Encoder}->encode($s, $check);
-    Encode::_utf8_off($enc) if is_utf8($enc); # workaround for RT #35120
+    Encode::_utf8_off($enc); # workaround for RT #35120
     $enc;
 }
 
@@ -769,8 +731,9 @@ sub encoded_header_len($$$;) {
     my $enclen;
     if ($encoding eq 'Q') {
         $enclen = _enclen_Q($s);
-    } elsif ($encoding eq 'S' and _resolve_S($s) eq 'Q') {
-	$enclen = _enclen_Q($s);
+    } elsif ($encoding eq "S") {
+        my ($b, $q) = (_enclen_B($s), _enclen_Q($s));
+	$enclen = ($b < $q)? $b: $q;
     } else { # "B"
         $enclen = _enclen_B($s);
     }
@@ -794,19 +757,6 @@ sub _enclen_Q($;$) {
     length($s) + scalar(@o) * 2;
 }
 
-sub _resolve_S($) {
-    my $s = shift;
-    my $in_body = shift;
-    my $e;
-    if ($in_body) {
-	$e = scalar(() = $s =~ m{[^-\t\r\n !*+/0-9A-Za-z]}g);
-	return (length($s) + 8 < $e * 6) ? 'BASE64' : 'QUOTED-PRINTABLE';
-    } else {
-	$e = scalar(() = $s =~ m{[^- !*+/0-9A-Za-z]}g);
-	return (length($s) + 8 < $e * 6) ? 'B' : 'Q';
-    }
-}
-
 =item $charset->header_encode(STRING [, OPTS])
 
 =item header_encode STRING, CHARSET [, OPTS]
@@ -867,7 +817,11 @@ sub header_encode {
             $cset = 'US-ASCII';
         }
     } elsif ($enc eq 'S') {
-	$enc = _resolve_S($encoded);
+	if (_enclen_B($encoded) < _enclen_Q($encoded)) {
+	    $enc = 'B';
+	} else {
+	    $enc = 'Q';
+	}
     } elsif ($enc !~ /^[BQ]$/) {
         $enc = 'B';
     }
@@ -1142,7 +1096,7 @@ sub recommended ($;$;$;$) {
 
 Unicode/multibyte support flag.
 Non-empty string will be set when Unicode and multibyte support is enabled.
-Currently, this flag will be non-empty on Perl 5.7.3 or later and
+Currently, this flag will be non-empty on Perl 5.8.0 or later and
 empty string on earlier versions of Perl.
 
 =back
@@ -1212,7 +1166,7 @@ L<http://hatuka.nezumi.nu/repos/MIME-Charset/>.
 
 =over 4
 
-=item *
+=item o
 
 new() method returns an object when CHARSET argument is not specified.
 
@@ -1222,22 +1176,22 @@ new() method returns an object when CHARSET argument is not specified.
 
 =over 4
 
-=item *
+=item o
 
 Restrict characters in encoded-word according to RFC 2047 section 5 (3).
 This also affects return value of encoded_header_len() method.
 
 =back
 
-=item Release 1.008.2
+=item Release 1.008
 
 =over 4
 
-=item *
+=item o
 
 body_encoding() method may also returns C<"S">.
 
-=item *
+=item o
 
 Return value of body_encode() method for UTF-8 may include
 C<"QUOTED-PRINTABLE"> encoding item that in earlier versions was fixed to
@@ -1257,7 +1211,7 @@ Hatuka*nezumi - IKEDA Soji <hatuka(at)nezumi.nu>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2006-2011 Hatuka*nezumi - IKEDA Soji.
+Copyright (C) 2006-2009 Hatuka*nezumi - IKEDA Soji.
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
